//+------------------------------------------------------------------+
//|                                               MultiUseSignal.mq5 |
//|                                Copyright 2012, Victor A.Kirillin |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include <Expert\Expert.mqh>
//--- available signals
#include <Expert\Signal\SignalAC.mqh>
//--- available trailing
#include <Expert\Trailing\TrailingFixedPipsWithSignal.mqh>
//--- available money management
#include <Expert\Money\MoneyFixedLotWithSignal.mqh>
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
//--- inputs for expert
input string Expert_Title                            ="MultiUseSignal"; // Document name
ulong        Expert_MagicNumber                      =9516;             // 
bool         Expert_EveryTick                        =false;            // 
//--- inputs for main signal
input int    Signal_ThresholdOpen                    =10;               // Signal threshold value to open [0...100]
input int    Signal_ThresholdClose                   =10;               // Signal threshold value to close [0...100]
input double Signal_PriceLevel                       =0.0;              // Price level to execute a deal
input double Signal_StopLevel                        =50.0;             // Stop Loss level (in points)
input double Signal_TakeLevel                        =50.0;             // Take Profit level (in points)
input int    Signal_Expiration                       =4;                // Expiration of pending orders (in bars)
input double Signal_AC_Weight                        =1.0;              // Accelerator Oscillator Weight [0...1.0]
//--- inputs for trailing
input int    Trailing_FixedPipsWithSignal_StopLevel  =30;               // Stop Loss trailing level (in points)
input int    Trailing_FixedPipsWithSignal_ProfitLevel=50;               // Take Profit trailing level (in points)
//--- inputs for money
input double Money_FixLotWithSignal_Percent          =10.0;             // Percent
input double Money_FixLotWithSignal_Lots             =0.1;              // Fixed volume
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
CExpert ExtExpert;
//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- Initializing expert
   if(!ExtExpert.Init(Symbol(),Period(),Expert_EveryTick,Expert_MagicNumber))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing expert");
      ExtExpert.Deinit();
      return(-1);
     }
//--- Creating signal
   CExpertSignal *signal=new CExpertSignal;
   if(signal==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating signal");
      ExtExpert.Deinit();
      return(-2);
     }
//---
   ExtExpert.InitSignal(signal);
   signal.ThresholdOpen(Signal_ThresholdOpen);
   signal.ThresholdClose(Signal_ThresholdClose);
   signal.PriceLevel(Signal_PriceLevel);
   signal.StopLevel(Signal_StopLevel);
   signal.TakeLevel(Signal_TakeLevel);
   signal.Expiration(Signal_Expiration);
//--- Creating filter CSignalAC
   CSignalAC *filter0=new CSignalAC;
   if(filter0==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter0");
      ExtExpert.Deinit();
      return(-3);
     }
   signal.AddFilter(filter0);
//--- Set filter parameters
   filter0.Weight(Signal_AC_Weight);
//--- Creation of trailing object
   TrailingFixedPipsWithSignal *trailing=new TrailingFixedPipsWithSignal;
   if(trailing==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating trailing");
      ExtExpert.Deinit();
      return(-4);
     }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert.InitTrailing(trailing))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing trailing");
      ExtExpert.Deinit();
      return(-5);
     }
//--- !!!!!!!!!!!!!!!!!!!!!!    !!!!!!!!!!!!!!!!!!!!!!
   trailing.Signal(signal);
//--- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//--- Set trailing parameters
   trailing.StopLevel(Trailing_FixedPipsWithSignal_StopLevel);
   trailing.ProfitLevel(Trailing_FixedPipsWithSignal_ProfitLevel);
//--- Creation of money object
   CMoneyFixedLotWithSignal *money=new CMoneyFixedLotWithSignal;
   if(money==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating money");
      ExtExpert.Deinit();
      return(-6);
     }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert.InitMoney(money))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing money");
      ExtExpert.Deinit();
      return(-7);
     }
//--- !!!!!!!!!!!!!!!!!!!!!!    !!!!!!!!!!!!!!!!!!!!!!
   money.Signal(signal);
//--- !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
//--- Set money parameters
   money.Percent(Money_FixLotWithSignal_Percent);
   money.Lots(Money_FixLotWithSignal_Lots);
//--- Check all trading objects parameters
   if(!ExtExpert.ValidationSettings())
     {
      //--- failed
      ExtExpert.Deinit();
      return(-8);
     }
//--- Tuning of all necessary indicators
   if(!ExtExpert.InitIndicators())
     {
      //--- failed
      printf(__FUNCTION__+": error initializing indicators");
      ExtExpert.Deinit();
      return(-9);
     }
//--- ok
   return(0);
  }
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   ExtExpert.Deinit();
  }
//+------------------------------------------------------------------+
//| "Tick" event handler function                                    |
//+------------------------------------------------------------------+
void OnTick()
  {
   ExtExpert.OnTick();
  }
//+------------------------------------------------------------------+
//| "Trade" event handler function                                   |
//+------------------------------------------------------------------+
void OnTrade()
  {
   ExtExpert.OnTrade();
  }
//+------------------------------------------------------------------+
//| "Timer" event handler function                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
   ExtExpert.OnTimer();
  }
//+------------------------------------------------------------------+
